@@ -62,6 +62,8 @@ XFCE_PANEL_PLUGIN_REGISTER ( create_plugin_control);
 void
 make_menu (plugin_data *pd);
 
+static void
+add_edit_clicked (GtkButton *buttonn, gpointer data);
 
 
 /* This is the timeout function that repeats the alarm */
@@ -157,123 +159,124 @@ update_function (gpointer data)
   list = pd->alarm_list;
 
   while (list){
-	  alrm = (alarm_t *) list->data;
-	  if(alrm->timer_on){
+      alrm = (alarm_t *) list->data;
+      if(alrm->timer_on){
 
-	  elapsed_sec = (gint) g_timer_elapsed(alrm->timer, NULL);
+      elapsed_sec = (gint) g_timer_elapsed(alrm->timer, NULL);
   /* If countdown is not over, update tooltip */
-	  if (elapsed_sec < alrm->timeout_period_in_sec) {
-		  remaining = alrm->timeout_period_in_sec - elapsed_sec;
+      if (elapsed_sec < alrm->timeout_period_in_sec) {
+          remaining = alrm->timeout_period_in_sec - elapsed_sec;
 
-		  if (remaining >= 3600)
-			tiptext = g_strdup_printf (_("%dh %dm %ds left"), remaining / 3600,
-									   (remaining % 3600) / 60, remaining % 60);
-		  else if (remaining >= 60)
-			tiptext = g_strdup_printf (_("%dm %ds left"), remaining / 60,
-									   remaining % 60);
-		  else
-			tiptext = g_strdup_printf (_("%ds left"), remaining);
+          if (remaining >= 3600)
+            tiptext = g_strdup_printf (_("%dh %dm %ds left"), remaining / 3600,
+                                       (remaining % 3600) / 60, remaining % 60);
+          else if (remaining >= 60)
+            tiptext = g_strdup_printf (_("%dm %ds left"), remaining / 60,
+                                       remaining % 60);
+          else
+            tiptext = g_strdup_printf (_("%ds left"), remaining);
 
-		  if (alrm->is_paused)
-			{
-			  temp = g_strconcat (tiptext, _(" (Paused)"), NULL);
-			  g_free (tiptext);
-			  tiptext = temp;
-			}
-		  if(alrm->timeout_period_in_sec < min_remaining_time){
-			  min_remaining_time = alrm->timeout_period_in_sec;
-			  gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pd->pbar),
-					  1.0 - ((gdouble) elapsed_sec) / alrm->timeout_period_in_sec);
-		  }
-	  callAgain =  TRUE;
-	  }else{
-			  gchar *command;
+          if (alrm->is_paused)
+            {
+              temp = g_strconcat (tiptext, _(" (Paused)"), NULL);
+              g_free (tiptext);
+              tiptext = temp;
+            }
+          if(alrm->timeout_period_in_sec < min_remaining_time){
+              min_remaining_time = alrm->timeout_period_in_sec;
+              gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pd->pbar),
+                      1.0 - ((gdouble) elapsed_sec) / alrm->timeout_period_in_sec);
+          }
+      callAgain =  TRUE;
+      }else{
+              gchar *command;
 
-			  /* Countdown is over, stop timer and free resources */
-			  if (alrm->timer)
-				  g_timer_destroy(alrm->timer);
-			  alrm->timer = NULL;
-			  /* Disable tooltips, reset pbar */
-			  gtk_widget_set_tooltip_text (GTK_WIDGET (pd->base), "");
-			  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (pd->pbar), 0);
+              /* Countdown is over, stop timer and free resources */
+              if (alrm->timer)
+                  g_timer_destroy(alrm->timer);
+              alrm->timer = NULL;
+              /* Disable tooltips, reset pbar */
+              gtk_widget_set_tooltip_text (GTK_WIDGET (pd->base), "");
+              gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (pd->pbar), 0);
 
-			  alrm->timeout = 0;
-			  alrm->timer_on = FALSE;
+              alrm->timeout = 0;
+              alrm->timer_on = FALSE;
 
-			  /* If an alarm command is set, it overrides the default (if any) */
-			  if (strlen(alrm->command)>0)
-				command = g_strdup(alrm->command);
-			  else if (pd->use_global_command)
-				command = g_strdup (pd->global_command);
-			  else
-				command = g_strdup("");
+              /* If an alarm command is set, it overrides the default (if any) */
+              if (strlen(alrm->command)>0)
+                command = g_strdup(alrm->command);
+              else if (pd->use_global_command)
+                command = g_strdup (pd->global_command);
+              else
+                command = g_strdup("");
 
-			  if ((strlen(command) == 0) || !pd->nowin_if_alarm) {
-				gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (pd->pbar), 0);
+              if ((strlen(command) == 0) || !pd->nowin_if_alarm) {
+                gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (pd->pbar), 0);
 
-				/* Display the name of the alarm when the countdown ends */
-				dialog_message = g_strdup_printf(_("Beeep! :) \nTime is up for the alarm %s."), alrm->name);
-				dialog_title = g_strdup_printf("Xfce4 Timer Plugin: %s", alrm->name);
+                /* Display the name of the alarm when the countdown ends */
+                dialog_message = g_strdup_printf(_("Beeep! :) \nTime is up for the alarm %s."), alrm->name);
+                dialog_title = g_strdup_printf(_("Timer %s"), alrm->name);
 
-				dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
-						GTK_MESSAGE_WARNING, GTK_BUTTONS_NONE, "%s", dialog_message);
+                dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
+                        GTK_MESSAGE_WARNING, GTK_BUTTONS_NONE, "%s", dialog_message);
 
-				gtk_window_set_title((GtkWindow *) dialog, dialog_title);
+                gtk_window_set_keep_above((GtkWindow *) dialog, TRUE);
 
-				gtk_dialog_add_button((GtkDialog *) dialog, _("Close"), 0);
-				gtk_dialog_add_button((GtkDialog *) dialog, _("Rerun the timer"), 1);
+                gtk_window_set_title((GtkWindow *) dialog, dialog_title);
 
-				g_signal_connect(dialog, "response",
-						G_CALLBACK(dialog_response),
-						alrm);
+                gtk_dialog_add_button((GtkDialog *) dialog, _("Close"), 0);
+                gtk_dialog_add_button((GtkDialog *) dialog, _("Rerun the timer"), 1);
 
-				g_free(dialog_title);
-				g_free(dialog_message);
+                g_signal_connect(dialog, "response",
+                        G_CALLBACK(dialog_response),
+                        alrm);
 
-				gtk_widget_show(dialog);
-			}
+                g_free(dialog_title);
+                g_free(dialog_message);
 
-			if (strlen(command) > 0) {
+                gtk_widget_show(dialog);
+            }
 
-				g_spawn_command_line_async(command, NULL);
+            if (strlen(command) > 0) {
 
-				if (pd->repeat_alarm_command) {
-					alrm->is_repeating = TRUE;
-					alrm->rem_repetitions = pd->repetitions;
-					if (alrm->repeat_timeout != 0)
-						g_source_remove(alrm->repeat_timeout);
-					alrm->repeat_timeout = g_timeout_add(pd->repeat_interval * 1000, repeat_alarm, alrm);
-				} else {
-					if (command)
-						g_free(command);
-					command = NULL;
-				}
-			}
+                g_spawn_command_line_async(command, NULL);
 
-			//Check if alarm is recurring after it's finished and destroyed; if yes then start it again.
-			if(alrm->is_recurring){
-				start_timer(pd,alrm);
-			}
+                if (pd->repeat_alarm_command) {
+                    alrm->is_repeating = TRUE;
+                    alrm->rem_repetitions = pd->repetitions;
+                    if (alrm->repeat_timeout != 0)
+                        g_source_remove(alrm->repeat_timeout);
+                    alrm->repeat_timeout = g_timeout_add(pd->repeat_interval * 1000, repeat_alarm, alrm);
+                } else {
+                    if (command)
+                        g_free(command);
+                    command = NULL;
+                }
+            }
 
-		  }
-		  //tiptext = g_strconcat("\t", tiptext, NULL);
-		  temp = g_strconcat(alrm->name ,"\t", tiptext, NULL);
-		  g_free(tiptext);
-		  tiptext = temp;
-		  //if not first
-		  if(firstActiveTimer){
-		  	firstActiveTimer = FALSE;
-		  }else{
-			temp = g_strconcat("\n", tiptext, NULL);
-			g_free(tiptext);
-			tiptext = temp;
-		  }
-		  temp = g_strconcat(finalTipText, tiptext, NULL);
-		  g_free(finalTipText);
-		  finalTipText = temp;
-		}
-		list = g_list_next (list);
-	  }
+          // Start next alarm if configured
+          if (alrm->next_alarm)
+            start_timer(pd, alrm->next_alarm);
+
+          }
+          //tiptext = g_strconcat("\t", tiptext, NULL);
+          temp = g_strconcat(alrm->name ,"\t", tiptext, NULL);
+          g_free(tiptext);
+          tiptext = temp;
+          //if not first
+          if(firstActiveTimer){
+            firstActiveTimer = FALSE;
+          }else{
+            temp = g_strconcat("\n", tiptext, NULL);
+            g_free(tiptext);
+            tiptext = temp;
+          }
+          temp = g_strconcat(finalTipText, tiptext, NULL);
+          g_free(finalTipText);
+          finalTipText = temp;
+        }
+        list = g_list_next (list);
+      }
   gtk_widget_set_tooltip_text (GTK_WIDGET(pd->base), finalTipText);
   g_free(tiptext);
   g_free(finalTipText);
@@ -312,11 +315,9 @@ static void
 start_timer (plugin_data *pd, alarm_t* alrm)
 {
 
-  gchar temp[8];
   gint cur_h, cur_m, cur_s;
   gint timeout_period;
-  GTimeVal timeval;
-  struct tm *current;
+  GDateTime *current;
 
   /* Empty timer list-> Nothing to do. alrm=0, though */
   if (alrm == NULL)
@@ -331,14 +332,11 @@ start_timer (plugin_data *pd, alarm_t* alrm)
   if (!alrm->is_countdown)
     {
 
-      g_get_current_time (&timeval);
-      current = localtime ((time_t *) &timeval.tv_sec);
-      strftime (temp, 7, "%H", current);
-      cur_h = atoi (temp);
-      strftime (temp, 7, "%M", current);
-      cur_m = atoi (temp);
-      strftime (temp, 7, "%S", current);
-      cur_s = atoi (temp);
+      current = g_date_time_new_now_local ();
+      cur_h = g_date_time_get_hour   (current);
+      cur_m = g_date_time_get_minute (current);
+      cur_s = g_date_time_get_second (current);
+      g_date_time_unref (current);
 
       timeout_period = (alrm->time) * 60 - ((60 * cur_h + cur_m) * 60 + cur_s);
 
@@ -350,7 +348,7 @@ start_timer (plugin_data *pd, alarm_t* alrm)
   /* Else 'alrm->selected->time' already gives the countdown period in seconds */
   else
     {
-	  alrm->is_countdown = TRUE;
+      alrm->is_countdown = TRUE;
       timeout_period = alrm->time;
     }
 
@@ -375,26 +373,26 @@ start_timer (plugin_data *pd, alarm_t* alrm)
 static void
 start_stop_callback (GtkWidget* menuitem, gpointer list)
 {
-	  GList *listitem = (GList *) list;
-	  plugin_data *pd;
-	  alarm_t *alrm;
+      GList *listitem = (GList *) list;
+      plugin_data *pd;
+      alarm_t *alrm;
 
-	  alrm = (alarm_t *) listitem->data;
-	  pd = (plugin_data *) alrm->pd;
+      alrm = (alarm_t *) listitem->data;
+      pd = (plugin_data *) alrm->pd;
 
   /* If counting down, we stop the timer and free the resources */
   if (alrm->timer_on)
     {
 
-	  if(alrm->timer)
-		 g_timer_destroy(alrm->timer);
-	  if(alrm->timeout)
-		 g_source_remove(alrm->timeout);
+      if(alrm->timer)
+         g_timer_destroy(alrm->timer);
+      if(alrm->timeout)
+         g_source_remove(alrm->timeout);
 
-	    alrm->timer = NULL;
-	    alrm->timeout = 0;
-	    alrm->is_paused = FALSE;
-	    alrm->timer_on = FALSE;
+        alrm->timer = NULL;
+        alrm->timeout = 0;
+        alrm->is_paused = FALSE;
+        alrm->timer_on = FALSE;
 
       /* Disable tooltips, reset pbar */
       gtk_widget_set_tooltip_text (GTK_WIDGET (pd->base), "");
@@ -425,7 +423,7 @@ pause_resume_selected (GtkWidget* menuitem, gpointer data)
     }
   else
     {
-	  alrm->is_paused = TRUE;
+      alrm->is_paused = TRUE;
       g_timer_stop (alrm->timer);
     }
 }
@@ -451,6 +449,26 @@ pbar_clicked (GtkWidget *pbar, GdkEventButton *event, gpointer data)
     gtk_menu_popdown (GTK_MENU (pd->menu));
 }
 
+/* This function help to remove left padding from menu */
+/* src:https://gist.github.com/andreldm/62ecee0d87cde8ec5f3d0e36e404511c */
+static GtkWidget*
+create_menu_item (const gchar *str, const char *icon_name)
+{
+  GtkWidget *mi, *label, *box, *image;
+
+  mi = gtk_menu_item_new ();
+  label = gtk_label_new (str);
+  image = gtk_image_new_from_icon_name (icon_name ? icon_name : "", GTK_ICON_SIZE_BUTTON);
+
+  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
+  gtk_widget_set_halign (label, GTK_ALIGN_START);
+
+  gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 6);
+  gtk_container_add (GTK_CONTAINER (mi), box);
+
+  return mi;
+}
 
 
 /* This function generates the popup menu */
@@ -459,7 +477,7 @@ make_menu (plugin_data *pd)
 {
   GList *list = NULL;
   alarm_t *alrm;
-  GtkWidget *menuitem;
+  GtkWidget *menu_item;
   gchar *itemtext;
 
   /* Destroy the existing one */
@@ -467,71 +485,65 @@ make_menu (plugin_data *pd)
     gtk_widget_destroy (pd->menu);
 
   pd->menu = gtk_menu_new ();
+ /* remove left padding from menu, see also create_menu_item () */
+  gtk_menu_set_reserve_toggle_size (GTK_MENU (pd->menu), FALSE);
 
   list = pd->alarm_list;
 
+  menu_item = create_menu_item (_("Add new alarm"), "xfce4-timer-plugin");
+  gtk_menu_shell_append (GTK_MENU_SHELL( pd->menu ), menu_item);
+
+  g_signal_connect( G_OBJECT(menu_item), "activate",
+                    G_CALLBACK (add_edit_clicked), pd);
+
   while (list)
     {
       /* Run through the list, read name and timer period info */
 
       alrm = (alarm_t *) list->data;
-
       itemtext = g_strdup_printf ("%s (%s)", alrm->name, alrm->info);
 
-      /* The selected timer is always active */
-      if(alrm->timer_on){
-		menuitem=gtk_menu_item_new_with_label(itemtext);
-		gtk_menu_shell_append(GTK_MENU_SHELL(pd->menu),menuitem);
-		gtk_widget_set_sensitive(GTK_WIDGET(menuitem),FALSE);
+      menu_item = create_menu_item (itemtext, NULL);
+      gtk_menu_shell_append (GTK_MENU_SHELL(pd->menu),menu_item);
+      g_free (itemtext);
 
-		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem),TRUE);
+      if (alrm->timer_on) {
+        gtk_widget_set_sensitive (GTK_WIDGET(menu_item), FALSE);
 
-		/* Pause menu item */
-		if(!alrm->is_paused && alrm->is_countdown) {
-			menuitem=gtk_menu_item_new_with_label(_("Pause timer"));
+        /* Pause menu item */
+        if (!alrm->is_paused && alrm->is_countdown) {
+            menu_item = create_menu_item (_("Pause timer"), "media-playback-pause");
+            gtk_menu_shell_append (GTK_MENU_SHELL(pd->menu), menu_item);
+            g_signal_connect (G_OBJECT(menu_item),"activate",
+                              G_CALLBACK(pause_resume_selected), alrm);
+        }
+        /* If the alarm is paused, the only option is to resume or stop */
+        else if (alrm->is_paused) {
+            menu_item = create_menu_item(_("Resume timer"), "media-playback-start");
+            gtk_menu_shell_append (GTK_MENU_SHELL(pd->menu), menu_item);
+            g_signal_connect (G_OBJECT(menu_item), "activate",
+                              G_CALLBACK(pause_resume_selected), alrm);
+        }
 
-			gtk_menu_shell_append   (GTK_MENU_SHELL(pd->menu),menuitem);
-			g_signal_connect    (G_OBJECT(menuitem),"activate",
-					G_CALLBACK(pause_resume_selected),alrm);
-		}
-		/* If the alarm is paused, the only option is to resume or stop */
-		else if (alrm->is_paused) {
-			menuitem=gtk_menu_item_new_with_label(_("Resume timer"));
+        menu_item = create_menu_item(_("Stop timer"), "media-playback-stop");
+        gtk_menu_shell_append (GTK_MENU_SHELL(pd->menu), menu_item);
+        g_signal_connect (G_OBJECT(menu_item), "activate",
+                          G_CALLBACK(start_stop_callback), list);
+      } else {
+        g_signal_connect (G_OBJECT(menu_item), "activate",
+                          G_CALLBACK (timer_selected), list);
+        /* disable alarm menu entry if repeating command */
+        if (alrm->is_repeating)
+          gtk_widget_set_sensitive(GTK_WIDGET(menu_item),FALSE);
+      }
 
-			gtk_menu_shell_append (GTK_MENU_SHELL(pd->menu),menuitem);
-			g_signal_connect  (G_OBJECT(menuitem),"activate",
-					G_CALLBACK(pause_resume_selected),alrm);
-			gtk_widget_show(menuitem);
-
-			gtk_widget_show(menuitem);
-			gtk_widget_show(pd->menu);
-		}
-
-		menuitem=gtk_menu_item_new_with_label(_("Stop timer"));
-
-		gtk_menu_shell_append (GTK_MENU_SHELL(pd->menu),menuitem);
-		g_signal_connect  (G_OBJECT(menuitem),"activate",
-				G_CALLBACK(start_stop_callback),list);
-
-
-	}else{
-		menuitem=gtk_menu_item_new_with_label(itemtext);
-		gtk_menu_shell_append(GTK_MENU_SHELL(pd->menu),menuitem);
-		g_signal_connect  (G_OBJECT(menuitem),"activate",
-				G_CALLBACK (timer_selected), list);
-		/* disable alarm menu entry if repeating command */
-		if(alrm->is_repeating)
-		  gtk_widget_set_sensitive(GTK_WIDGET(menuitem),FALSE);
-	}
-
-    g_free (itemtext);
-    list = list->next;
-    if(list){
-	  /* Horizontal line (empty item) */
-	  menuitem=gtk_separator_menu_item_new();
-	  gtk_menu_shell_append(GTK_MENU_SHELL(pd->menu),menuitem);
-	}
-  }
+      list = list->next;
+      if (list) {
+        /* Horizontal line (empty item) */
+        menu_item = gtk_separator_menu_item_new();
+        gtk_menu_shell_append (GTK_MENU_SHELL(pd->menu),menu_item);
+      }
+    }
 
   gtk_widget_show_all (pd->menu);
 }
@@ -620,7 +632,7 @@ static void
 ok_edit (GtkButton *button, gpointer data)
 {
   alarm_data *adata = (alarm_data *) data;
-  GtkTreeIter iter;
+  GtkTreeIter iter, next_alarm_iter;
   gint t1, t2, t3, t;
   gchar *timeinfo = NULL;
   GList *list;
@@ -642,8 +654,13 @@ ok_edit (GtkButton *button, gpointer data)
           gtk_entry_get_text (GTK_ENTRY (adata->command)));
       alrm->is_countdown = gtk_toggle_button_get_active (
           GTK_TOGGLE_BUTTON (adata->rb1));
-      alrm->is_recurring = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(adata->
-                                 recur_cb));
+
+      if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (adata->next_alarm), &next_alarm_iter))
+        gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (adata->next_alarm)),
+                            &next_alarm_iter, 0, &alrm->next_alarm, -1);
+      else
+        alrm->next_alarm = NULL;
+
       alrm->is_auto_start = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(adata->
                                  autostart_cb));
 
@@ -736,7 +753,8 @@ alarmdialog_alarmtime_toggled (GtkButton *button, gpointer data)
 
 
 /**
- * Callback to the Add button in options window
+ * Callback to the Add and Edit buttons in options window
+ * Callback to the "Add new alarm" menu entry
  * Creates the Add window
  **/
 static void
@@ -750,6 +768,10 @@ add_edit_clicked (GtkButton *buttonn, gpointer data)
   GtkEntry *name, *command;
   GtkSpinButton *timeh, *timem, *times, *time_h, *time_m;
   GtkRadioButton *rb1, *rb2;
+  GtkListStore *next_alarm_list;
+  GList *alarm_iter;
+  GtkWidget *next_alarm;
+  GtkCellRenderer *next_alarm_cell;
   GtkWidget *hbox, *vbox, *button;
   alarm_data *adata = g_new0 (alarm_data, 1);
   gint time;
@@ -775,9 +797,6 @@ add_edit_clicked (GtkButton *buttonn, gpointer data)
   adata->pd = pd;
 
   /* Set title */
-  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-  gtk_container_add (GTK_CONTAINER (dialog), vbox);
-  
   box = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
   gtk_container_set_border_width (GTK_CONTAINER (box), 2);
 
@@ -871,16 +890,35 @@ add_edit_clicked (GtkButton *buttonn, gpointer data)
   gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (command), TRUE, TRUE, 0);
   adata->command = command;
 
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
+  gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (hbox), TRUE, TRUE, 0);
+
+  label = (GtkLabel *) gtk_label_new (_("Alarm to run:"));
+  gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (label), FALSE, FALSE, 0);
+
+  /* 'next alarm' combo box to set subsequent alarm (if any)
+   * Separate list store created to add 'empty' item for no next alarm */
+  next_alarm_list = gtk_list_store_new (2, G_TYPE_POINTER, G_TYPE_STRING);
+  gtk_list_store_insert_with_values (next_alarm_list, &iter, -1, 0, NULL, 1, "", -1);
+  alarm_iter = pd->alarm_list;
+  while (alarm_iter)
+  {
+    alrm = alarm_iter->data;
+    gtk_list_store_insert_with_values (next_alarm_list, &iter, -1, 0, alrm, 1, alrm->name, -1);
+    alarm_iter = alarm_iter->next;
+  }
+  next_alarm = gtk_combo_box_new_with_model (GTK_TREE_MODEL (next_alarm_list));
+  g_object_unref (next_alarm_list);
+  next_alarm_cell = gtk_cell_renderer_text_new();
+  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (next_alarm), next_alarm_cell, TRUE);
+  gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (next_alarm), next_alarm_cell, "text", 1);
+  gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (next_alarm), FALSE, FALSE, 0);
+  adata->next_alarm = next_alarm;
+
   /****************/
 
   gtk_box_pack_start (GTK_BOX (vbox), gtk_separator_new (GTK_ORIENTATION_HORIZONTAL), FALSE, FALSE, 6);
 
-  //add recurring alarm check button
-  button = gtk_check_button_new_with_label(_("Recurring alarm"));
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button), FALSE);
-  gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
-  adata->recur_cb = button;
-
    //add alarm autostart check button
   button = gtk_check_button_new_with_label(_("Auto start when plugin loads"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), FALSE);
@@ -899,14 +937,13 @@ add_edit_clicked (GtkButton *buttonn, gpointer data)
 
   button = gtk_button_new_with_label (_("Accept"));
   gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
-  if (GTK_WIDGET (buttonn) == pd->buttonadd)
+  if (GTK_WIDGET (buttonn) == pd->buttonedit)
+    g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (ok_edit), adata);
+  else /* Add button or "Add new alarm" menu entry*/
     g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (ok_add), adata);
-  else
-    g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (ok_edit),
-                      adata);
 
-  /* If this is the add window, we're done */
-  if (GTK_WIDGET (buttonn) == pd->buttonadd)
+  /* If this is not edit window (add button or add menu entry) we're done */
+  if (GTK_WIDGET (buttonn) != pd->buttonedit)
     {
       gtk_window_set_title (GTK_WINDOW (dialog), _("Add new alarm"));
       gtk_widget_show_all (GTK_WIDGET (dialog));
@@ -926,9 +963,9 @@ add_edit_clicked (GtkButton *buttonn, gpointer data)
       gtk_entry_set_text (GTK_ENTRY (name), alrm->name);
       gtk_entry_set_text (GTK_ENTRY (command), alrm->command);
 
-      //load settings
-	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(adata->recur_cb),alrm->is_recurring);
-	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(adata->autostart_cb),alrm->is_auto_start);
+      gtk_combo_box_set_active (GTK_COMBO_BOX (next_alarm),
+                                g_list_index (pd->alarm_list, alrm->next_alarm) + 1);
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(adata->autostart_cb),alrm->is_auto_start);
 
       time = alrm->time;
 
@@ -963,7 +1000,8 @@ remove_clicked (GtkButton *button, gpointer data)
   GtkTreeIter iter;
   GtkTreeModel *model;
   GtkTreeSelection *select;
-  GList *list;
+  GList *list, *alarm_iter;
+  alarm_t *alrm;
 
   /* Get the selected row */
   select = gtk_tree_view_get_selection (GTK_TREE_VIEW (pd->tree));
@@ -976,6 +1014,16 @@ remove_clicked (GtkButton *button, gpointer data)
 
   gtk_tree_model_get (model, &iter, 0, &list, -1);
 
+  // If removed alarm is used as a 'next alarm' somwhere, unreference it
+  alarm_iter = pd->alarm_list;
+  while (alarm_iter)
+  {
+    alrm = alarm_iter->data;
+    if (alrm->next_alarm == list->data)
+      alrm->next_alarm = NULL;
+    alarm_iter = alarm_iter->next;
+  }
+
   if (pd->selected == list)
     {
       pd->alarm_list = g_list_delete_link (pd->alarm_list, list);
@@ -1092,6 +1140,9 @@ update_pbar_orientation (XfcePanelPlugin *plugin, plugin_data *pd)
       gtk_progress_bar_set_inverted (GTK_PROGRESS_BAR (pd->pbar), TRUE);
       gtk_widget_set_halign (GTK_WIDGET (pd->pbar), GTK_ALIGN_CENTER);
       gtk_widget_set_hexpand (GTK_WIDGET (pd->pbar), TRUE);
+
+      gtk_widget_set_size_request (GTK_WIDGET (plugin), -1,
+                                   xfce_panel_plugin_get_size (plugin));
     }
   else
     {
@@ -1104,6 +1155,9 @@ update_pbar_orientation (XfcePanelPlugin *plugin, plugin_data *pd)
       gtk_progress_bar_set_inverted (GTK_PROGRESS_BAR (pd->pbar), FALSE);
       gtk_widget_set_valign (GTK_WIDGET (pd->pbar), GTK_ALIGN_CENTER);
       gtk_widget_set_hexpand (GTK_WIDGET (pd->pbar), FALSE);
+
+      gtk_widget_set_size_request (GTK_WIDGET (plugin),
+                                   xfce_panel_plugin_get_size (plugin), -1);
     }
 }
 
@@ -1118,6 +1172,19 @@ orient_change (XfcePanelPlugin *plugin, GtkOrientation orient, plugin_data *pd)
 
 
 
+/* Callback for size change of panel */
+static gboolean
+size_changed (XfcePanelPlugin *plugin, gint size, plugin_data *pd)
+{
+  if (xfce_panel_plugin_get_orientation (plugin) == GTK_ORIENTATION_HORIZONTAL)
+    gtk_widget_set_size_request (GTK_WIDGET (plugin), -1, size);
+  else
+    gtk_widget_set_size_request (GTK_WIDGET (plugin), size, -1);
+
+  return TRUE;
+}
+
+
 /**
  * Loads the settings and alarm list from a keyfile, saves the
  * alarm list in the linked list pd->alarm_list 
@@ -1128,13 +1195,15 @@ load_settings (plugin_data *pd)
   gchar groupname[8];
   const gchar *timerstring;
   gint groupnum, time;
-  gboolean is_cd, is_recur, autostart;
+  gboolean is_cd, autostart;
   alarm_t *alrm;
+  GList *alarm_iter;
   XfceRc *rc;
+  gchar* rc_path;
 
-  if (g_file_test (pd->configfile, G_FILE_TEST_EXISTS))
+  if ((rc_path = xfce_panel_plugin_lookup_rc_file (pd->base)))
     {
-      rc = xfce_rc_simple_open (pd->configfile, TRUE);
+      rc = xfce_rc_simple_open (rc_path, TRUE);
 
       if (rc != NULL)
         {
@@ -1162,11 +1231,16 @@ load_settings (plugin_data *pd)
               is_cd = xfce_rc_read_bool_entry (rc, "is_countdown", TRUE);
               alrm->is_countdown = is_cd;
 
-              is_recur=xfce_rc_read_bool_entry(rc,"is_recur",FALSE);
-			  alrm->is_recurring = is_recur;
+              alrm->next_alarm_groupnum = xfce_rc_read_int_entry (rc, "timernext", -1);
+              /* If 'timernext' not specified, fallback to checking previously
+               * used boolean setting 'is_recur' telling if alarm is self-recurring
+               * NOTE: remove in version ?.?? */
+              if (alrm->next_alarm_groupnum == -1)
+                alrm->next_alarm_groupnum =
+                  xfce_rc_read_bool_entry(rc, "is_recur", FALSE) ? groupnum : -1;
 
-			  autostart=xfce_rc_read_bool_entry(rc,"autostart",FALSE);
-			  alrm->is_auto_start = autostart;
+              autostart=xfce_rc_read_bool_entry(rc,"autostart",FALSE);
+              alrm->is_auto_start = autostart;
 
               time = xfce_rc_read_int_entry (rc, "time", 0);
               alrm->time = time;
@@ -1181,6 +1255,16 @@ load_settings (plugin_data *pd)
 
           pd->count = groupnum;
 
+          /* Once all rc groups (i.e. alarm configs) have been read, setup
+           * references to 'next alarm's based on 'groupnum' read from rc */
+          alarm_iter = pd->alarm_list;
+          while (alarm_iter)
+          {
+            alrm = alarm_iter->data;
+            alrm->next_alarm = g_list_nth_data(pd->alarm_list, alrm->next_alarm_groupnum);
+            alarm_iter = alarm_iter->next;
+          }
+
           /* Read other options */
           if (xfce_rc_has_group (rc, "others"))
             {
@@ -1209,6 +1293,8 @@ load_settings (plugin_data *pd)
           xfce_rc_close (rc);
         }
     }
+
+  g_free (rc_path);
 }
 
 
@@ -1223,7 +1309,7 @@ save_settings (XfcePanelPlugin *plugin, plugin_data *pd)
   alarm_t *alrm;
   FILE *conffile;
   XfceRc *rc;
-  gchar *file, *contents = NULL;
+  gchar *file;
 
   if (!(file = xfce_panel_plugin_save_location (plugin, TRUE)))
     return;
@@ -1243,6 +1329,18 @@ save_settings (XfcePanelPlugin *plugin, plugin_data *pd)
   if (!rc)
     return;
 
+  /* Assign groupnum to each alarm in the order they will be written to rc.
+   * These groupnums will be saved whenever 'next alarm' is set
+   * ('next alarm' is a pointer and saving it directly is meaningless). */
+  row_count = 0;
+  list = pd->alarm_list;
+  while (list)
+  {
+    alrm = list->data;
+    alrm->next_alarm_groupnum = row_count++;
+    list = list->next;
+  }
+
   list = pd->alarm_list;
 
   row_count = 0;
@@ -1265,9 +1363,10 @@ save_settings (XfcePanelPlugin *plugin, plugin_data *pd)
 
       xfce_rc_write_bool_entry (rc, "is_countdown", alrm->is_countdown);
 
-      xfce_rc_write_bool_entry(rc,"is_recur",alrm->is_recurring);
+      if (alrm->next_alarm)
+        xfce_rc_write_int_entry (rc, "timernext", alrm->next_alarm->next_alarm_groupnum);
 
-	  xfce_rc_write_bool_entry(rc,"autostart",alrm->is_auto_start);
+      xfce_rc_write_bool_entry(rc,"autostart",alrm->is_auto_start);
 
 
       row_count++;
@@ -1284,18 +1383,7 @@ save_settings (XfcePanelPlugin *plugin, plugin_data *pd)
   xfce_rc_write_int_entry (rc, "repeat_interval", pd->repeat_interval);
   xfce_rc_close (rc);
 
-  conffile = fopen (pd->configfile, "w");
-
-  /* We backup a permanent copy, which we'll use to load settings */
-  if (conffile && g_file_get_contents (file, &contents, NULL, NULL))
-    {
-      fputs (contents, conffile);
-      fclose (conffile);
-    }
-
   g_free (file);
-  if (contents)
-    g_free (contents);
 }
 
 
@@ -1322,23 +1410,20 @@ plugin_free (XfcePanelPlugin *plugin, plugin_data *pd)
   list = pd->alarm_list;
 
   while (list){
-	alrm = (alarm_t *) list->data;
-	/* remove timeouts */
-	if (alrm->timeout!=0) g_source_remove(alrm->timeout);
-	if (alrm->repeat_timeout!=0) g_source_remove(alrm->repeat_timeout);
+    alrm = (alarm_t *) list->data;
+    /* remove timeouts */
+    if (alrm->timeout!=0) g_source_remove(alrm->timeout);
+    if (alrm->repeat_timeout!=0) g_source_remove(alrm->repeat_timeout);
 
-	if(alrm->timer)
-	  g_timer_destroy(alrm->timer);
+    if(alrm->timer)
+      g_timer_destroy(alrm->timer);
 
-	list = g_list_next (list);
+    list = g_list_next (list);
   }
 
   if (pd->global_command)
     g_free (pd->global_command);
 
-  if (pd->configfile)
-    g_free (pd->configfile);
-
   if (pd->liststore)
     {
       gtk_list_store_clear (pd->liststore);
@@ -1352,8 +1437,6 @@ plugin_free (XfcePanelPlugin *plugin, plugin_data *pd)
 
   /* free the plugin data structure */
   g_free (pd);
-
-  gtk_main_quit ();
 }
 
 
@@ -1466,7 +1549,7 @@ plugin_create_options (XfcePanelPlugin *plugin, plugin_data *pd)
   xfce_panel_plugin_block_menu (plugin);
 
   header = xfce_titled_dialog_new_with_buttons (
-      _("Xfce4 Timer Options"),
+      _("Timer Options"),
       GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
       GTK_DIALOG_DESTROY_WITH_PARENT, _("Close"), GTK_RESPONSE_OK, NULL);
 
@@ -1657,9 +1740,11 @@ plugin_create_options (XfcePanelPlugin *plugin, plugin_data *pd)
 static void
 show_about_window (XfcePanelPlugin *plugin, plugin_data *pd)
 {
-  GdkPixbuf *icon;
   const gchar *author[] =
-    { "Kemal Ilgar Eroğlu <ilgar_eroglu@yahoo.com>", NULL };
+    { "Kemal Ilgar Eroğlu <ilgar_eroglu@yahoo.com>, 2005-2018",
+      "The Xfce development team, 2019-2023",
+      NULL
+    };
   const gchar *translators =
       "Mohammad Alhargan <malham1@hotmail.com> \n\
 Marcos Antonio Alvarez Costales <marcoscostales@gmail.com> \n\
@@ -1696,19 +1781,15 @@ Muhammad Ali Makki <makki.ma@gmail.com>\n\
 Hunt Xu <huntxu@live.cn>\n\
 Cheng-Chia Tseng <pswo10680@gmail.com>\n";
 
-  icon = xfce_panel_pixbuf_from_source ("xfce4-timer-plugin", NULL, 48);
   gtk_show_about_dialog (
-      NULL, "title", _("About xfce4-timer-plugin"), "logo", icon, "license",
+      NULL, "title", _("About Timer plugin"), "logo-icon-name", "xfce4-timer-plugin", "license",
       xfce_get_license_text (XFCE_LICENSE_TEXT_GPL), "version", PACKAGE_VERSION,
       "program-name", PACKAGE_NAME, "comments",
       _("A plugin to define countdown timers or alarms at given times."),
       "website",
       "https://docs.xfce.org/panel-plugins/xfce4-timer-plugin",
-      "copyright", _("Copyright (c) 2005-2018\n"), "authors", author,
+      "copyright", _("Copyright (c) 2005-2023\n"), "authors", author,
       "translator-credits", translators, NULL);
-
-  if (icon)
-    g_object_unref (G_OBJECT (icon));
 }
 
 
@@ -1722,7 +1803,6 @@ Cheng-Chia Tseng <pswo10680@gmail.com>\n";
 static void
 create_plugin_control (XfcePanelPlugin *plugin)
 {
-  gchar *filename, *pathname;
   plugin_data *pd = g_new0 (plugin_data, 1);
   GList *list = NULL;
   alarm_t *alrm;
@@ -1758,12 +1838,6 @@ create_plugin_control (XfcePanelPlugin *plugin)
 
   g_object_ref (pd->liststore);
 
-  filename = xfce_panel_plugin_save_location (pd->base, TRUE);
-  pathname = g_path_get_dirname (filename);
-  pd->configfile = g_strconcat (pathname, "/XfceTimer.rc", NULL);
-  g_free (filename);
-  g_free (pathname);
-
   load_settings (pd);
   pd->selected = pd->alarm_list;
   //Check if an alarm is auto start to start it at creation
@@ -1796,7 +1870,7 @@ create_plugin_control (XfcePanelPlugin *plugin)
   g_signal_connect (plugin, "orientation-changed", G_CALLBACK (orient_change),
                     pd);
 
-  g_signal_connect (plugin, "size-changed", G_CALLBACK (gtk_true), NULL);
+  g_signal_connect (plugin, "size-changed", G_CALLBACK (size_changed), pd);
 
   xfce_panel_plugin_menu_show_configure (plugin);
   g_signal_connect (plugin, "configure-plugin",
